@@ -159,10 +159,7 @@ def _manage_status_change(task_status, task, comment):
             if task_status["is_feedback_request"]:
                 new_data["end_date"] = date_helpers.get_utc_now_datetime()
 
-            if (
-                task_status["short_name"] == "wip"
-                and task["real_start_date"] is None
-            ):
+            if task_status["is_wip"] and task["real_start_date"] is None:
                 new_data["real_start_date"] = datetime.datetime.now(
                     datetime.timezone.utc
                 )
@@ -252,24 +249,25 @@ def _run_status_automation(automation, task, person_id):
                         task["id"]
                     )
                 )
-                preview_files = (
-                    preview_files_service.get_preview_files_for_revision(
-                        preview_file["task_id"], preview_file["revision"]
-                    )
-                )
-
-                for preview_file in preview_files:
-                    new_preview_file = (
-                        tasks_service.add_preview_file_to_comment(
-                            new_comment["id"],
-                            new_comment["person_id"],
-                            task_to_update["id"],
+                if preview_file is not None:
+                    preview_files = (
+                        preview_files_service.get_preview_files_for_revision(
+                            preview_file["task_id"], preview_file["revision"]
                         )
                     )
 
-                    preview_files_service.copy_preview_file_in_another_one(
-                        preview_file["id"], new_preview_file["id"]
-                    )
+                    for preview_file in preview_files:
+                        new_preview_file = (
+                            tasks_service.add_preview_file_to_comment(
+                                new_comment["id"],
+                                new_comment["person_id"],
+                                task_to_update["id"],
+                            )
+                        )
+
+                        preview_files_service.copy_preview_file_in_another_one(
+                            preview_file["id"], new_preview_file["id"]
+                        )
 
     elif automation["out_field_type"] == "ready_for":
         try:
@@ -329,7 +327,11 @@ def new_comment(
     add_attachments_to_comment(comment, files)
     events.emit(
         "comment:new",
-        {"comment_id": comment["id"], "task_id": task_id},
+        {
+            "comment_id": comment["id"],
+            "task_id": task_id,
+            "task_status_id": task_status_id,
+        },
         project_id=task["project_id"],
     )
     return comment
