@@ -1,5 +1,5 @@
 from sqlalchemy.orm import aliased
-from sqlalchemy import func, or_
+from sqlalchemy import func, or_, and_
 
 from zou.app import config
 from zou.app.models.comment import Comment
@@ -7,7 +7,7 @@ from zou.app.models.entity import Entity
 from zou.app.models.entity_type import EntityType
 from zou.app.models.notification import Notification
 from zou.app.models.person import Person
-from zou.app.models.project import Project
+from zou.app.models.project import Project, ProjectPersonLink
 from zou.app.models.project_status import ProjectStatus
 from zou.app.models.subscription import Subscription
 from zou.app.models.search_filter import SearchFilter
@@ -15,6 +15,7 @@ from zou.app.models.search_filter_group import SearchFilterGroup
 from zou.app.models.task import Task
 from zou.app.models.task_type import TaskType
 
+
 from zou.app.services import (
     assets_service,
     custom_actions_service,
@@ -35,7 +36,7 @@ from zou.app.services.exception import (
     NotificationNotFoundException,
     WrongParameterException,
 )
-from zou.app.utils import cache, fields, permissions
+from zou.app.utils import cache, fields, permissions, events
 
 
 def clear_filter_cache(user_id=None):
@@ -77,7 +78,8 @@ def build_open_project_filter():
     """
     Query filter for project to retrieve only open projects.
     """
-    return ProjectStatus.name.in_(("Active", "open", "Open"))
+    open_status = projects_service.get_open_status()
+    return Project.project_status_id == open_status["id"]
 
 
 def build_related_projects_filter():
@@ -85,13 +87,8 @@ def build_related_projects_filter():
     Query filter for project to retrieve open projects of which the user
     is part of the team.
     """
-    projects = (
-        Project.query.join(ProjectStatus)
-        .filter(build_team_filter())
-        .filter(build_open_project_filter())
-        .all()
-    )
-    project_ids = [project.id for project in projects]
+    projects = related_projects()
+    project_ids = [project["id"] for project in projects]
     if len(project_ids) > 0:
         return Project.id.in_(project_ids)
     else:
@@ -101,16 +98,30 @@ def build_related_projects_filter():
 def related_projects():
     """
     Return all projects related to current user: open projects of which the user
-    is part of the team.
+    is part of the team as dicts.
     """
+    persons_service.get_current_user()
+    projects = related_projects_raw()
+    return Project.serialize_list(projects)
+
+
+def related_projects_raw():
+    """
+    Return all projects related to current user: open projects of which the user
+    is part of the team as models.
+    """
+    current_user = persons_service.get_current_user()
     projects = (
-        Project.query.join(Task)
-        .join(ProjectStatus)
-        .filter(build_team_filter())
+        Project.query.join(
+            ProjectStatus, Project.project_status_id == ProjectStatus.id
+        )
+        .join(ProjectPersonLink, Project.id == ProjectPersonLink.project_id)
+        .filter(ProjectPersonLink.person_id == current_user["id"])
         .filter(build_open_project_filter())
+        .distinct()
         .all()
     )
-    return Project.serialize_list(projects)
+    return projects
 
 
 def get_todos():
@@ -158,7 +169,7 @@ def get_tasks_for_entity(entity_id):
     """
     query = (
         Task.query.join(Project)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .filter(Task.entity_id == entity_id)
         .filter(build_assignee_filter())
         .filter(build_open_project_filter())
@@ -175,7 +186,7 @@ def get_task_types_for_entity(entity_id):
     query = (
         TaskType.query.join(Task)
         .join(Project)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .filter(Task.entity_id == entity_id)
         .filter(build_assignee_filter())
         .filter(build_open_project_filter())
@@ -193,7 +204,7 @@ def get_assets_for_asset_type(project_id, asset_type_id):
         Entity.query.join(EntityType)
         .join(Project)
         .join(Task, Task.entity_id == Entity.id)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .filter(EntityType.id == asset_type_id)
         .filter(Project.id == project_id)
         .filter(build_assignee_filter())
@@ -212,7 +223,7 @@ def get_asset_types_for_project(project_id):
         EntityType.query.join(Entity, Entity.entity_type_id == EntityType.id)
         .join(Task, Task.entity_id == Entity.id)
         .join(Project)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .filter(Project.id == project_id)
         .filter(build_assignee_filter())
         .filter(build_open_project_filter())
@@ -236,7 +247,7 @@ def get_sequences_for_project(project_id):
         .join(Task, Task.entity_id == Shot.id)
         .join(EntityType, EntityType.id == Entity.entity_type_id)
         .join(Project, Project.id == Entity.project_id)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .filter(Shot.entity_type_id == shot_type["id"])
         .filter(Entity.entity_type_id == sequence_type["id"])
         .filter(Project.id == project_id)
@@ -263,7 +274,7 @@ def get_project_episodes(project_id):
         .join(Shot, Shot.parent_id == Sequence.id)
         .join(Task, Task.entity_id == Shot.id)
         .join(Project, Project.id == Entity.project_id)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .filter(Shot.entity_type_id == shot_type["id"])
         .filter(Sequence.entity_type_id == sequence_type["id"])
         .filter(Entity.entity_type_id == episode_type["id"])
@@ -283,7 +294,7 @@ def get_shots_for_sequence(sequence_id):
     query = (
         Entity.query.join(Task)
         .join(Project)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .join(EntityType)
         .filter(Entity.entity_type_id == shot_type["id"])
         .filter(Entity.parent_id == sequence_id)
@@ -303,7 +314,7 @@ def get_scenes_for_sequence(sequence_id):
     query = (
         Entity.query.join(Task)
         .join(Project)
-        .join(ProjectStatus)
+        .join(ProjectStatus, Project.project_status_id == ProjectStatus.id)
         .join(EntityType)
         .filter(Entity.entity_type_id == scene_type["id"])
         .filter(Entity.parent_id == sequence_id)
@@ -318,15 +329,19 @@ def get_open_projects(name=None):
     """
     Get all open projects for which current user is part of the team.
     """
-    query = Project.query.join(ProjectStatus).filter(
-        build_open_project_filter()
-    )
+    query = Project.query.join(
+        ProjectStatus, Project.project_status_id == ProjectStatus.id
+    ).filter(build_open_project_filter())
 
     if name is not None:
         query = query.filter(Project.name == name)
 
     if not permissions.has_admin_permissions():
-        query = query.filter(build_team_filter())
+        current_user = persons_service.get_current_user()
+        query = query.join(
+            ProjectPersonLink, Project.id == ProjectPersonLink.project_id
+        )
+        query = query.filter(ProjectPersonLink.person_id == current_user["id"])
 
     for_client = False
     vendor_departments = None
@@ -351,9 +366,16 @@ def get_open_project_ids():
 
 def get_projects(name=None):
     """
-    Get all projects for which current user has a task assigned.
+    Get all projects for which current user is part of the team.
     """
-    query = Project.query.join(ProjectStatus).filter(build_team_filter())
+    current_user = persons_service.get_current_user()
+    query = (
+        Project.query.join(
+            ProjectStatus, Project.project_status_id == ProjectStatus.id
+        )
+        .join(ProjectPersonLink, Project.id == ProjectPersonLink.project_id)
+        .filter(ProjectPersonLink.person_id == current_user["id"])
+    )
 
     if name is not None:
         query = query.filter(Project.name == name)
@@ -395,8 +417,10 @@ def check_person_access(person_id):
     """
     Return True if user is an admin or is matching given person id.
     """
-    current_user = persons_service.get_current_user()
-    if permissions.has_admin_permissions() or current_user["id"] == person_id:
+    if (
+        permissions.has_admin_permissions()
+        or persons_service.get_current_user()["id"] == person_id
+    ):
         return True
     else:
         raise permissions.PermissionDenied
@@ -417,7 +441,7 @@ def check_belong_to_project(project_id):
 
 def has_project_access(project_id):
     """
-    Return true if current user is a manager or has a task assigned for this
+    Return true if current user is an admin or has a task assigned for this
     project.
     """
     return permissions.has_admin_permissions() or check_belong_to_project(
@@ -488,6 +512,62 @@ def check_task_access(task_id):
     return True
 
 
+def check_task_action_access(task_id):
+    """
+    Return true if current user can have access to a task action.
+    """
+    task = tasks_service.get_task(task_id, relations=True)
+    is_allowed = False
+    if permissions.has_admin_permissions():
+        is_allowed = True
+    elif check_belong_to_project(task["project_id"]):
+        if (
+            permissions.has_manager_permissions()
+            or permissions.has_client_permissions()
+        ):
+            is_allowed = True
+        else:
+            user = persons_service.get_current_user(relations=True)
+            is_allowed = user["id"] in task["assignees"]
+            if not is_allowed and permissions.has_supervisor_permissions():
+                is_allowed = (
+                    user["departments"] == []
+                    or tasks_service.get_task_type(task["task_type_id"])[
+                        "department_id"
+                    ]
+                    in user["departments"]
+                )
+
+    if not is_allowed:
+        raise permissions.PermissionDenied
+    return is_allowed
+
+
+def check_supervisor_project_task_type_access(project_id, task_type_id):
+    """
+    Return true if current user can have access to a task type.
+    """
+    is_allowed = False
+    if permissions.has_admin_permissions() or (
+        permissions.has_manager_permissions()
+        and check_belong_to_project(project_id)
+    ):
+        is_allowed = True
+    elif permissions.has_supervisor_permissions() and check_belong_to_project(
+        project_id
+    ):
+        user = persons_service.get_current_user(relations=True)
+        is_allowed = (
+            user["departments"] == []
+            or tasks_service.get_task_type(task_type_id)["department_id"]
+            in user["departments"]
+        )
+
+    if not is_allowed:
+        raise permissions.PermissionDenied
+    return is_allowed
+
+
 def check_comment_access(comment_id):
     """
     Return true if current user can have access to a comment.
@@ -625,38 +705,6 @@ def check_supervisor_task_access(task, new_data={}):
     return is_allowed
 
 
-def check_supervisor_schedule_item_access(schedule_item, new_data={}):
-    """
-    Return true if current user is a manager and has a task assigned related
-    to the project of this task or is a supervisor and can modify data accorded
-    to his departments
-    """
-    is_allowed = False
-    if permissions.has_admin_permissions() or (
-        permissions.has_manager_permissions()
-        and check_belong_to_project(schedule_item["project_id"])
-    ):
-        is_allowed = True
-    elif permissions.has_supervisor_permissions() and check_belong_to_project(
-        schedule_item["project_id"]
-    ):
-        user_departments = persons_service.get_current_user(relations=True)[
-            "departments"
-        ]
-        if (
-            user_departments == []
-            or tasks_service.get_task_type(schedule_item["task_type_id"])[
-                "department_id"
-            ]
-            in user_departments
-        ):
-            is_allowed = True
-
-    if not is_allowed:
-        raise permissions.PermissionDenied
-    return is_allowed
-
-
 def check_metadata_department_access(entity, new_data={}):
     """
     Return true if current user is a manager and has a task assigned for this
@@ -797,10 +845,7 @@ def check_task_department_access_for_unassign(task_id, person_id=None):
                     or task_type["department_id"] in user["departments"]
                 )
             )
-            or (
-                user["id"] in task.get("assignees", [])
-                and person_id == user["id"]
-            )
+            or (user["id"] in task["assignees"] and person_id == user["id"])
         )
     )
     if not is_allowed:
@@ -1105,6 +1150,7 @@ def get_user_filter_groups(current_user_id):
             )
         )
         .filter(or_(build_open_project_filter(), Project.id == None))
+        .order_by(SearchFilterGroup.created_at.desc())
         .all()
     )
 
@@ -1288,6 +1334,22 @@ def update_notification(notification_id, read):
         id=notification_id, person_id=current_user["id"]
     )
     notification.update({"read": read})
+    if read:
+        events.emit(
+            "notification:read",
+            {
+                "person_id": current_user["id"],
+                "notification_id": notification_id,
+            },
+        )
+    else:
+        events.emit(
+            "notification:unread",
+            {
+                "person_id": current_user["id"],
+                "notification_id": notification_id,
+            },
+        )
     return notification.serialize()
 
 
@@ -1301,9 +1363,6 @@ def get_unread_notifications_count(notification_id=None):
     ).count()
 
 
-from sqlalchemy import and_, func
-
-
 def get_last_notifications(
     notification_id=None,
     after=None,
@@ -1422,12 +1481,15 @@ def get_last_notifications(
                 ),
                 None,
             )
-            reply_mentions = reply.get("mentions", []) or []
-            reply_department_mentions = (
-                reply.get("department_mentions", []) or []
-            )
             if reply is not None:
                 reply_text = reply["text"]
+                reply_mentions = reply.get("mentions", []) or []
+                reply_department_mentions = (
+                    reply.get("department_mentions", []) or []
+                )
+            else:
+                reply_mentions = []
+                reply_department_mentions = []
 
         if role == "client" and is_current_user_artist:
             comment_text = ""
@@ -1485,7 +1547,7 @@ def mark_notifications_as_read():
 
     db.session.execute(update_stmt)
     db.session.commit()
-
+    events.emit("notification:all-read", {"person_id": current_user["id"]})
     return True
 
 
